--[[        ]]
--[[ Locals ]]
--[[        ]]

local RL                = AceLibrary("Roster-2.1")
local L                 = AceLibrary("AceLocale-2.2"):new("Litmus")

local NameWidth = 90
local MissingWidth = 50
local Padding = 4

local testing = false;
local testDuration = 0;

--[[          ]]
--[[ CreateUI ]]
--[[          ]]

function Litmus:BuildDisplay()

	self.frames = self.frames or {}
	self.frames.units = self.frames.units or {}
	
	-- Create our main frame to hold all our gui elements
	local f = CreateFrame("Frame", "MainFrame", UIParent)
		f:SetMovable(true)
--~ 		f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", self.db.profile.x, self.db.profile.y)
		f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", self.db.profile.x, self.db.profile.y)
		f:SetClampedToScreen(true)
	-- hopefully this will stop the frame from capturing mouse clicks
		f:EnableMouse(false)
	self.frames.main = f
   
	-- Create a button to hold our title
	-- We'll use a button so we can move our frame by clicking on it
	local title = CreateFrame("Button", "Title", self.frames.main)
		title:EnableMouse(true)                            
		title:SetHeight(16)
		title:SetPoint("TOP", self.frames.main, "TOP", 0, 0)
		title:SetScript("OnMouseUp", self.StopMoveFrame)
		title:SetScript("OnMouseDown",self.StartMoveFrame)
	self.frames.title = title
   
	-- Create the text for our title button
	local text = title:CreateFontString(nil, "ARTWORK")
		text:SetFontObject(GameFontHighlightSmall)
		text:SetJustifyH("CENTER")
		text:SetJustifyV("BOTTOM")
		text:SetHeight(12)
		text:SetPoint("TOP", self.frames.title, "TOP", 0, -5)
	self.frames.titletext = text
   
	local bg = CreateFrame("Frame", "Background", self.frames.main)
		bg:SetFrameLevel(0)
		bg:SetPoint("TOP",self.frames.title,"BOTTOM",0,0)
		bg:Show()
	self.frames.bg = bg

	local texture = self.frames.bg:CreateTexture(nil, "LOW")
		texture:SetPoint("CENTER", self.frames.bg, "CENTER",0,0)
	self.frames.bg.texture = texture
   
   
	-- Finish setting up our frames/bars
	self:UpdateFrame()
	self:UpdateBars()
   
	-- Set the scale for the entire GUI. Make sure to do this last
	self.frames.main:SetScale(self.db.profile.DisplayScale)
end

--[[             ]]
--[[ UpdateFrame ]]
--[[             ]]

function Litmus:UpdateFrame()
   
	-- Adjust frame size if name is placed inside the bar
	local currNameWidth = NameWidth
	if (Litmus.db.profile.NamePosition == L["InsideBar"]) then
		currNameWidth = 0
	end
	
	-- Adjust frame size if missing mana is hidden or placed inside the bar
	local currMissingWidth = 0;
	if (Litmus.db.profile.ManaPosition == L["OutsideBar"]) then
		currMissingWidth = MissingWidth + Padding
	end
	
	
	-- Sum width and height
	local totalWidth = self.db.profile.BarWidth + currNameWidth + currMissingWidth + (Padding * 3) + Padding + self.db.profile.BarHeight -- padding for frame to name, name to bar, bar to mana/frame
	local totalHeight = self.db.profile.ScanUnits * (self.db.profile.BarHeight + self.db.profile.BarSpacing) + self.db.profile.BarSpacing + 5 + 5
	
	-- Main frame
	self.frames.main:SetWidth(totalWidth)
	self.frames.main:SetHeight(totalHeight)
	
	
	-- Frame background
	self.frames.bg:SetAlpha(self.db.profile.FrameAlpha)
	self.frames.bg:SetBackdrop({
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", tile = true, 
		tileSize = 32,
		insets = {left = 5, right = 5, top = 5, bottom = 5},
	})
	self.frames.bg:SetWidth(self.frames.main:GetWidth())
	self.frames.bg:SetHeight(self.frames.main:GetHeight())
   
	-- Frame background color
	self.frames.bg.texture:SetTexture(
		self.db.profile.FrameColor.r,
		self.db.profile.FrameColor.g,
		self.db.profile.FrameColor.b
	)
	self.frames.bg.texture:SetWidth(self.frames.bg:GetWidth()-10)
	self.frames.bg.texture:SetHeight(self.frames.bg:GetHeight()-10)
   
	-- Frame title
	self.frames.title:SetWidth(totalWidth)
   
	-- Frame title text
	self.frames.titletext:SetWidth(totalWidth)
	self.frames.titletext:SetText(self:GetMyTitle())
end

--[[            ]]
--[[ UpdateBars ]]
--[[            ]]

function Litmus:UpdateBars()
 	
	-- Hide all frame units
	for i in pairs(self.frames.units) do self.frames.units[i]:Hide() end
	
	for i = 1, self.db.profile.ScanUnits do
		local frame
		local name
		local bar
		local bagBG
		local missing
--[[
		local innervate
		local shadow
]]
		
		-- If we haven't created a frame for this unit position before, make it
		if self.frames.units[i] == nil then
			-- Create the gui items
			frame	= CreateFrame("Button", nil, self.frames.main)
			name	= frame:CreateFontString(nil, "ARTWORK")
			barBG	= CreateFrame("StatusBar", nil, frame)
			bar		= CreateFrame("StatusBar", nil, frame)
			missing	= frame:CreateFontString(nil,"ARTWORK")
			innervate = CreateFrame("Frame",nil,frame)
			groupnumber	= frame:CreateFontString(nil,"ARTWORK")
			
			-- Set up (mostly) static variables
			frame:Hide()
			frame:EnableMouse(true)
			frame:RegisterForClicks("LeftButtonUp", "RightButtonUp", "MiddleButtonUp", "Button4Up", "Button5Up")
			frame:SetParent(self.frames.main)
			
			name:SetFontObject(GameFontHighlightSmall)
			name:SetHeight(12)
			
			missing:SetFontObject(GameFontHighlightSmall)
			missing:SetHeight(12)
			
			barBG:SetMinMaxValues(0,100)
			barBG:SetFrameLevel(1)
			
			bar:SetMinMaxValues(0,100)
			bar:SetFrameLevel(2)
			
			local icon_i = innervate:CreateTexture(nil,"ARTWORK")
			icon_i:SetTexture("Interface\\Icons\\Spell_Nature_Lightning")
			icon_i:SetAllPoints(innervate)
			innervate.texture = icon_i
			
			innervate:SetFrameLevel(3)
			
			groupnumber:SetFontObject(GameFontHighlightSmall)
			groupnumber:SetHeight(12)
			
			-- Add it to our frames to display
			frame.Name = name
			frame.Bar = bar
			frame.BarBG = barBG
			frame.Missing = missing
			frame.Innervate = innervate
			frame.GroupNumber = groupnumber
			self.frames.units[i] = frame
			
		-- If we have, just get references to the items
		else
			frame	= self.frames.units[i]
			name	= self.frames.units[i].Name
			barBG	= self.frames.units[i].BarBG
			bar		= self.frames.units[i].Bar
			missing	= self.frames.units[i].Missing
			innervate	= self.frames.units[i].Innervate
			groupnumber	= self.frames.units[i].GroupNumber
		end
		
		-- See if user wants us to hide the missing mana part
		if (Litmus.db.profile.ManaPosition == L["Hidden"]) then
			missing:Hide()
		else
			missing:Show()
		end
		
		if Litmus.db.profile.DisplaySubGroup and UnitInRaid("player") then
--~ 		if Litmus.db.profile.DisplaySubGroup then
			groupnumber:Show()
		else
			groupnumber:Hide()
		end
		
		-- See if user wants us to hide the innervate part
--~ 		if not (Litmus.db.profile.ShowInnervate == L["Hidden"]) and UnitHasInnervate(i.unitid) then
--~ 		if not (Litmus.db.profile.ShowInnervate == L["Hidden"]) then
--~ 		Litmus:Print(i)
--~ 		Litmus:Print(Litmus.frames.units[i].HasInnervate)

--~ 		if Litmus.frames.units[i].HasInnervate then
--~ 			innervate:Show()
--~ 			Litmus:Print(" TRUE")
--~ 		else
--~ 			innervate:Hide()
--~ 			Litmus:Print(" false")
--~ 		end
		innervate:Show()
		
		-- We have to make sure we set the bar width to the total size, so every
		-- part of the bar can capture mouse clicks
		local currNameWidth = NameWidth
		if (Litmus.db.profile.NamePosition == L["InsideBar"]) then
			currNameWidth = 0
		end
		local currMissingWidth = 0;
		if (Litmus.db.profile.ManaPosition == L["OutsideBar"]) then
			currMissingWidth = MissingWidth + Padding
		end
		
		frame:SetWidth(self.db.profile.BarWidth + currNameWidth + currMissingWidth + Padding*2) 
		frame:SetHeight(self.db.profile.BarHeight)
		
		name:SetWidth(NameWidth)
		name:ClearAllPoints()
		
		missing:SetWidth(MissingWidth + Padding)
		missing:ClearAllPoints()
		-- If we're putting missing mana in bar, use the right side of the text as an anchor
		if (Litmus.db.profile.ManaPosition == L["InsideBar"]) then
			missing:SetJustifyH("RIGHT")
			missing:SetPoint("RIGHT", frame.BarBG, "RIGHT", -Padding/2, 0)
		-- Else, use the left side of the text as an anchor
		else
			missing:SetJustifyH("LEFT")
			missing:SetPoint("LEFT", frame.BarBG, "RIGHT", Padding, 0)
		end
		
		
		barBG:SetStatusBarTexture( "Interface\\AddOns\\Litmus\\images\\" .. Litmus.db.profile.BarTexture .. ".tga" )
		barBG:ClearAllPoints()
		barBG:SetWidth(self.db.profile.BarWidth)
		barBG:SetHeight(self.db.profile.BarHeight)
		
		
		bar:SetStatusBarTexture( "Interface\\AddOns\\Litmus\\images\\" .. Litmus.db.profile.BarTexture .. ".tga" )
		bar:ClearAllPoints()
		bar:SetWidth(self.db.profile.BarWidth)
		bar:SetHeight(self.db.profile.BarHeight)
		
		-- If we're putting the name in the bar, anchor to the left of the frame
		if (Litmus.db.profile.NamePosition == L["InsideBar"]) then
			name:SetPoint("LEFT", frame, "LEFT", Padding+Padding+Padding/2+self.db.profile.BarHeight, 0)
			name:SetJustifyH("LEFT")
			barBG:SetPoint("LEFT", frame, "LEFT", Padding+Padding+Padding/2+self.db.profile.BarHeight, 0)
			bar:SetPoint("LEFT", frame, "LEFT", Padding+Padding+Padding/2+self.db.profile.BarHeight, 0)
		-- Else, anchor to the right of the name text
		else
			name:SetPoint("LEFT", frame, "LEFT", Padding-Padding/2, 0)
			name:SetJustifyH("RIGHT")
			barBG:SetPoint("LEFT", frame.Name, "RIGHT", Padding*2+self.db.profile.BarHeight, 0)
			bar:SetPoint("LEFT", frame.Name, "RIGHT", Padding*2+self.db.profile.BarHeight, 0)
		end
		
		innervate:SetPoint("LEFT", frame.BarBG,-self.db.profile.BarHeight-Padding,0)
		innervate:SetWidth(self.db.profile.BarHeight+self.db.profile.BarSpacing)  -- Set These to whatever height/width is needed 
		innervate:SetHeight(self.db.profile.BarHeight+self.db.profile.BarSpacing) -- for your Texture
		
		groupnumber:SetPoint("CENTER", frame.BarBG, 0, 0)
		groupnumber:SetWidth(self.db.profile.BarWidth)
		groupnumber:SetHeight(self.db.profile.BarHeight)
		groupnumber:SetJustifyH("CENTER")
		groupnumber:SetJustifyV("CENTER")

		-- Set each bar's position below the previously created one
		if i == 1 then
			frame:SetPoint("TOPLEFT", self.frames.bg, "TOPLEFT", 0, -5-self.db.profile.BarSpacing)
		else
			self.frames.units[i]:SetPoint("TOPLEFT", self.frames.units[i-1], "BOTTOMLEFT", 0, -(self.db.profile.BarSpacing))
		end
	end
end

--[[                ]]
--[[ StartMoveFrame ]]
--[[                ]]

function Litmus:StartMoveFrame()
	if not Litmus.db.profile.Lock and arg1 == "LeftButton" then
		Litmus.frames.main:StartMoving()
		Litmus.frames.main.isMoving = true
	end
end

--[[               ]]
--[[ StopMoveFrame ]]
--[[               ]]

function Litmus:StopMoveFrame()
	if Litmus.frames.main.isMoving then
		Litmus.frames.main:StopMovingOrSizing()
		Litmus.db.profile.x = Litmus.frames.main:GetLeft()
		Litmus.db.profile.y = Litmus.frames.main:GetTop()
		Litmus.frames.main.isMoving = false
	end
end

--[[       ]]
--[[ Scale ]]
--[[       ]]

function Litmus:Scale()
	-- Get the current frame locs
	local x,y = self.db.profile.x,self.db.profile.y
	-- Multiply the locs by the current frame scale
	local s = self.frames.main:GetEffectiveScale()
	x,y = x*s,y*s
	-- Scale the frame
	self.frames.main:SetScale(self.db.profile.DisplayScale)
	-- Divide the locs by the new frame scale to get where we should be
	local s = self.frames.main:GetEffectiveScale()
	x,y = x/s,y/s
	-- Reset the frames position
	self.frames.main:ClearAllPoints()
	self.frames.main:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
--~ 	self.frames.main:SetPoint("CENTER", UIParent, "CENTER", x, y)
	-- Save the location
	self.db.profile.x = x
	self.db.profile.y = y
end

--[[            ]]
--[[ GetMyTitle ]]
--[[            ]]

function Litmus:GetMyTitle()
   if self.db.profile.FrameTitleOn then
      return L["Litmus"]
   else
      return ""
   end
end

--[[                 ]]
--[[ CheckVisibility ]]
--[[                 ]]

function Litmus:CheckVisibility()
	-- Always display
	if self.db.profile.DisplayWhen == L["Always"] then
		self:ShowGUI()
	-- We're in a party; show
	elseif self.db.profile.DisplayWhen == L["Party"] and GetNumPartyMembers() > 0 then
		self:ShowGUI()
	-- We're in a raid; show
	elseif self.db.profile.DisplayWhen == L["Raid"] and GetNumRaidMembers() > 0 then
		self:ShowGUI()
	-- We're either set to not-show, or we're not in a prereq party/raid
	else
		self:HideGUI()
	end
end

--[[        ]]
--[[ ShowUI ]]
--[[        ]]

function Litmus:ShowGUI()
	self.frames.main:Show()
	-- Register repeating AceEvent
	self:ScheduleRepeatingEvent("CheckUnitsID", self.CheckUnits, self.db.profile.ScanFrequency, self)
end

--[[        ]]
--[[ HideUI ]]
--[[        ]]

function Litmus:HideGUI()
	self.frames.main:Hide()
	-- Unegister AceEvent
	self:CancelScheduledEvent("CheckUnitsID")
end

--[[                 ]]
--[[ UpdateFrequency ]]
--[[                 ]]

function Litmus:UpdateFrequency()
	-- If we're in test mode, update the test speed.  The test will update the
	-- real scan frequency when its finished
	if testing then
		self:CancelScheduledEvent("CheckUnitsID")
		self:ScheduleRepeatingEvent("TestEventID", self.TestUpdate, self.db.profile.ScanFrequency, self)
	-- Otherwise, update the scan frequency like normal
	else
		self:CancelScheduledEvent("CheckUnitsID")
		self:ScheduleRepeatingEvent("CheckUnitsID", self.CheckUnits, self.db.profile.ScanFrequency, self)
	end
end

--[[              ]]
--[[ GetNameColor ]]
--[[              ]]

function Litmus:GetNameColor(class)
	-- If we're using class colors, and we found the class in wow's 
	-- class color table...
	-- if self.db.profile.ColorClass and RAID_CLASS_COLORS[class] then
	if self.db.profile.DisplayClassColor and RAID_CLASS_COLORS[class] then
		-- return the color
		local color = RAID_CLASS_COLORS[class]
		return color.r, color.g, color.b
	-- Otherwise, just return an off-white color
	elseif self.db.profile.DisplayClassColor and not RAID_CLASS_COLORS[class] then
		return 0.8,0.15,0.0
	else
		return self.db.profile.TextColor.r, self.db.profile.TextColor.g, self.db.profile.TextColor.b 
	end
end

--[[              ]]
--[[ FindBarColor ]]
--[[              ]]

function Litmus:FindBarColor(isBackground,unit)

	-- Default style: green health bar, black background
	if unit == nil or Litmus.db.profile.BarStyle == L["Default"] then
		if isBackground then
--~ 			return 0,0,1,Litmus.db.profile.BarAlpha/3
			return 0,0,1,Litmus.db.profile.BarAlpha/2
		else
			return 0.2,0.2,1,Litmus.db.profile.BarAlpha
		end
	-- Class style: bars colored by units class with a slightly darker background
	elseif Litmus.db.profile.BarStyle == L["Class"] then
		local color = RAID_CLASS_COLORS[unit.class]
		if isBackground then
			return color.r-0.4, color.g-0.4, color.b-0.4, Litmus.db.profile.BarAlpha
		else
			return color.r, color.g, color.b, Litmus.db.profile.BarAlpha
		end
	-- CTRaid style: red bar = player, blue = party member, green = everyone else
	else
		local color
		if UnitIsUnit(unit.unitid,"Player") then color = {r=1,g=0,b=0}
		elseif UnitInParty(unit.unitid) then color = {r=0,g=1,b=1}
		else color = {r=0,g=1,b=0}
		end
		
		if isBackground then
			return color.r-0.6, color.g-0.6, color.b-0.6, Litmus.db.profile.BarAlpha
		else
			return color.r, color.g, color.b, Litmus.db.profile.BarAlpha
		end
	end
end

--[[      ]]
--[[ Test ]]
--[[      ]]

function Litmus:Test()

	-- Cancel test
	if testing then
		self:TestCleanup()
		
	-- Start test
	else
		-- Set our global so we know
		testing = true
		-- Show if we're hidden
		self:ShowGUI()
		-- Make sure normal scanning is disabled
		self:CancelScheduledEvent("CheckUnitsID")
		-- Schedule test event
		self:ScheduleRepeatingEvent("TestEventID", self.TestUpdate, self.db.profile.ScanFrequency, self)
	end
	
end

--[[            ]]
--[[ TestUpdate ]]
--[[            ]]

function Litmus:TestUpdate()

	-- Make sure the test ends after a certain amount of time
	if testDuration < 100 then
	
		-- Populate display with dummy data
		testUnit = RL:GetUnitObjectFromUnit("player")
		for i=1,Litmus.db.profile.ScanUnits do
			Litmus.frames.units[i].unit = testUnit
			Litmus.frames.units[i].Name:SetText(testUnit.name)
			Litmus.frames.units[i].Name:SetTextColor(self:GetNameColor(testUnit.class))
			Litmus.frames.units[i].BarBG:SetStatusBarColor( Litmus:FindBarColor(true,testUnit) )
			Litmus.frames.units[i].Bar:SetStatusBarColor( Litmus:FindBarColor(false,testUnit) )
			Litmus.frames.units[i].Bar:SetValue(100-testDuration)
			Litmus.frames.units[i].Missing:SetText(" -" .. UnitManaMax(testUnit.unitid) - UnitMana(testUnit.unitid))
			Litmus.frames.units[i].Missing:SetTextColor(self.db.profile.TextColor.r, self.db.profile.TextColor.g, self.db.profile.TextColor.b)
			Litmus.frames.units[i].GroupNumber:SetText(i)
			Litmus.frames.units[i].GroupNumber:SetTextColor(self.db.profile.TextColor.r, self.db.profile.TextColor.g, self.db.profile.TextColor.b)
			Litmus.frames.units[i]:Show()
		end
			
		testDuration = testDuration + 2
	-- Call cleanup
	else
		self:TestCleanup()
	end
end

--[[             ]]
--[[ TestCleanup ]]
--[[             ]]

function Litmus:TestCleanup()
	-- Reset globals
	testing = false
	testDuration = 0;
	-- Cancel test event
	self:CancelScheduledEvent("TestEventID")
	-- Go back to normal
	self:CheckVisibility()
end
